/*
 * Server.java by Johann Betz
 * Date: 02.11.2009 
 */

import java.net.*;
import java.io.*;

public class Server {

    /**
     * The Main method. The webserver listens for connections and starts a
     * Thread (see serverThread.java) for every accepted connection. It takes a
     * port number as argument, when no portnumber is entered, the default port
     * 8080 is used.
     * 
     * @param args
     */
    public static void main(String[] args) {
        // defaultport 8080
        int port = 8080;
        if (args.length < 1) {
            System.out
                    .println("Please specify a valid portnumber, usage is: java server <portnumber>");
            System.out.println("Trying defaultport 8080");
        } else {
            try {
                port = Integer.parseInt(args[0]);
            } catch (NumberFormatException e) {
                System.out
                        .println("Please specify a valid portnumber, usage is: java server <portnumber>");
                System.out.println("Trying defaultport 8080");
            }
        }

        ServerSocket httpSock = null;
        try {
            // Create new Socket on the specified port
            httpSock = new ServerSocket(port);
            System.out.println("HTTP-Server ready!");

            // Infinity loop, listen for connections and start a thread if
            // accepted one
            while (true) {
                Socket s = null;
                s = httpSock.accept();
                System.out.println("Client accepted!");
                new serverThread(s);
            }
        } catch (UnknownHostException e) {
            System.err.println(e);
        } catch (IOException e) {
            System.err.println(e);
        }
        // Close the socket when the server is closed
        finally {
            if (httpSock != null) {
                try {
                    httpSock.close();
                } catch (IOException e) {

                }
            }
        }
    }
}
