/*
 * createIndex.java by Johann Betz
 * Date: 02.11.2009 
 */
import java.io.*;
import java.util.*;

/*
 * The class createIndex of Excercise 1
 * rewritten/written in Java
 * This can be executed without server.java and serverThread.java.
 * Added two methods for searching the invertedIndex which are used by
 * serverThread.java.. 
 */
public class createIndex {

    // List of Filenames
    private static List<String> FileNames = new ArrayList<String>();

    // Name of directory
    public static String DIRECTORYNAME = "d";

    private Properties p;
    FileWriter fw;
    BufferedWriter bw;
    String fs = System.getProperty("file.separator");

    public void getFileNames() {
        File directory = new File(DIRECTORYNAME);
        for (File file : directory.listFiles())
            FileNames.add(file.toString().replace(DIRECTORYNAME + "\\", ""));
    }

    /*
     * Removes the special chars of a string (a word)
     * 
     * @return String word - The word without special chars
     */
    public String removeSpecialChr(String word) {
        String specialchars[] = { "~", "*", "+", "?", "§", "(", ")", "{", "}",
                "]", "[", "=", "%", "/", "\\", "#", "|", "\n", "\r", "\f",
                "\\S", ">", "<", ",", "\"", ":", "-" };
        word = word.trim();
        for (int i = 0; i < specialchars.length; i++) {
            word = word.replace(specialchars[i], "");
        }

        return word;
    }

    /*
     * Check if an index already exists inside the invertedIndex.txt
     */
    public boolean CheckExistence(String indexToAdd) {
        String property = p.getProperty(indexToAdd, "null");
        if (property.equals("null"))
            return false;
        return true;
    }

    /*
     * Add an index to invertedIndex.txt
     */
    public void AddIndex(String filename, String indexToAdd) {
        boolean check = CheckExistence(indexToAdd);
        if (check) {
            String pr = p.getProperty(indexToAdd);
            if (!pr.contains(filename)) {
                pr += "," + filename;
                p.setProperty(indexToAdd, pr);
            }
        } else
            p.setProperty(indexToAdd, filename);
    }

    /*
     * createInd creates the inverted Index
     */
    public void createInd() throws IOException {
        loadIndex();
        Scanner scanner = null;
        getFileNames();
        for (int i = 0; i < FileNames.size(); i++) {
            String fname = FileNames.get(i);
            try {
                File f = new File(DIRECTORYNAME + fs + fname);
                scanner = new Scanner(f);
                while (scanner.hasNextLine()) {
                    String line = scanner.nextLine();
                    String[] word = line.split(" ");
                    for (int j = 0; j < word.length; j++) {
                        line = word[j].replace("-", "\n");
                        line = line.replace(".", "\n");
                        line = removeSpecialChr(line);
                        if (!line.equals(""))
                            AddIndex(fname, line);
                    }
                }
            } catch (Exception e) {
                System.out.println("An error occured:");
                e.printStackTrace();
            }
            scanner.close();
        }
    }

    /*
     * Stores the file reverseIndex.txt
     */
    public void storeIndex() throws IOException {
        FileOutputStream out = new FileOutputStream("invertedIndex.txt");
        p.store(out, "IndexFile: Index=Key(s)");
    }

    /**
     * Load indexFile.txt and create new property object
     * 
     * @throws IOException
     */
    public void loadIndex() throws IOException {
        FileInputStream in;
        try {
            in = new FileInputStream("invertedIndex.txt");
        } catch (FileNotFoundException e) {
            File f;
            f = new File("invertedIndex.txt");
            if (!f.exists()) {
                f.createNewFile();
            }
            f = null;
            in = new FileInputStream("invertedIndex.txt");
        }
        p = new Properties();
        p.load(in);
        in.close();
    }

    // Used to set the name of the directory from the userinput
    public void setDirectory(String dirname) {
        DIRECTORYNAME = dirname;
    }

    /*
     * Searches for the specified keywords keywords should be a String separated
     * by space, tab or newline
     */
    public String searchIndex(String keywords) {
        try {
            loadIndex();
        } catch (IOException e) {
            System.out.println("Could not open indexFile!");
        }
        StringTokenizer tokenizer = new StringTokenizer(keywords);
        String containingFiles[] = new String[tokenizer.countTokens()];
        int counter = 0;

        // read the results of the search for each keyword
        // after this containingFiles[0] contains a list
        // of files for the first keyword and so on...
        while (tokenizer.hasMoreTokens()) {
            String singlesearchword = tokenizer.nextToken();
            containingFiles[counter] = p.getProperty(singlesearchword, "null");
            if (containingFiles[counter].equals("null")) {
                return "No files matching all keywords could be found!";
            }
            System.out.println(containingFiles[counter]);
            counter++;
        }
        // Search now:
        String result = "", currentFile;
        StringTokenizer resultFiles = new StringTokenizer(containingFiles[0]
                .replace(",", " "));
        // If there is only one searchword, we don't have to intersect
        if (counter == 1) {
            result += containingFiles[0];
            return result;
        }
        // Intersect all the results to get only the results which contain all
        // keywords
        while (resultFiles.hasMoreTokens()) {
            for (int i = 1; i < counter; i++) {
                currentFile = resultFiles.nextToken();
                if (containingFiles[i].contains(currentFile)) {
                    result += currentFile + " ";
                }
            }

        }
        if (result.isEmpty()) {
            result = "No files matching all keywords could be found!";
        }
        return result;

    }

    // The main method is used only when executing createIndex alone.
    // It asks for a directorypath and creates the index for that directory.
    public static void main(String args[]) throws Exception {

        System.out.println("Enter directorypath for the documents:");
        BufferedReader bin = new BufferedReader(
                new InputStreamReader(System.in));
        String dname = bin.readLine();
        createIndex indexer = new createIndex();
        System.out
                .println("Start processing documents and creating the Index..");
        indexer.setDirectory(dname);
        indexer.createInd();
        indexer.storeIndex();
        System.out.println("Processing successful, invertedIndex.txt created");

    }
}