/*
 * serverThread.java by Johann Betz
 * Date: 02.11.2009 
 */

import java.io.*;
import java.net.*;
import java.util.*;

/*
 * The serverThread Class supplies a thread for every accepted
 * connection to the Server and handles Requests.
 */
public class serverThread extends Thread {
    BufferedReader bin = null;
    PrintWriter bout = null;

    Socket sock;

    // Start the thread for the specified socket
    public serverThread(Socket s) {
        sock = s;
        this.start();
    }

    /*
     * doSearch searches the invertedIndex file for specified keywords. As
     * parameter it takes the query which contains the keywords.
     * 
     * It returns an HTML-Code containing the Results.
     * 
     * @return content
     */
    public String doSearch(String query) {
        String searchstring, content = "";
        searchstring = query;
        int start = searchstring.indexOf("question=");
        int end = searchstring.indexOf("&", start);
        if (end > 0) {
            searchstring = searchstring.substring(start + "question=".length(),
                    end);
        } else {
            searchstring = searchstring.substring(start + "question=".length());
        }
        searchstring = searchstring.replace("+", " ");

        /*
         * Create an createIndex object, the constructor of createIndex() opens
         * up the file invertedIndex.txt. SearchIndex searches in
         * invertedIndex.txt for the searchstring. See createIndex.java for
         * details.
         */
        createIndex searcher = new createIndex();
        content += "You searched for: " + searchstring;
        content += "<br> Result was: <br> <br> "
                + searcher.searchIndex(searchstring);
        return content;
    }

    /*
     * doGetSentence extracts the integer specified in the request by
     * "repeat=xy" and repeats the sentence "This is a useful sentence!"
     * accordingly.
     */
    public String doGetSentence(String query) {
        int times = 1;
        String sentence = "", timesAsString;
        int start = query.indexOf("repeat=");
        int end = query.indexOf("&", start);

        timesAsString = query;

        if (end > 0) {
            timesAsString = timesAsString.substring(start + "repeat=".length(),
                    end);
        } else {
            timesAsString = timesAsString.substring(start + "repeat=".length());
        }
        try {
            times = Integer.parseInt(timesAsString);
        } catch (NumberFormatException e) {
            sentence = "Please enter a valid number after repeat=! E.g.: sentence?repeat=50 <br>";
        }
        for (int i = 0; i < times; i++) {
            sentence += "This is a useful sentence!<br>";
        }
        return sentence;
    }

    /*
     * getContent constructs the HTML Code for the reply of the Webserver. It
     * also calls doGetSentence and doSearch respectively.
     */
    public String getContent(URL theURL) {
        String htmlheader, htmlfooter, htmlsearch, content = "", requestedContent;
        requestedContent = "sentence";

        htmlheader = "<html><title>Search Engine and Sentence Repeater!</title><body>";
        htmlfooter = "</body></html>";
        htmlsearch = "<h1>Searchengine</h1>"
                + "<form action=\"search\" method=\"get\">"
                + "<input name=\"question\" type=\"text\" size=\"30\" />"
                + "<input type=\"submit\" />" + "</form> <br> <br>";

        // requestedContent contains the keyword specifying the returned content
        requestedContent = theURL.getPath();

        if (requestedContent.equals("/search")) {
            if (theURL.getQuery().length() > 0) {
                content = htmlsearch + doSearch(theURL.getQuery());
            }
        } else if (requestedContent.equals("/")) {
            content = htmlsearch;
        } else {
            if (theURL.getQuery().length() > 0) {
                content = doGetSentence(theURL.getQuery());
            }
        }

        // The content returned is htmlheader + content + htmlfooter
        return htmlheader + content + htmlfooter;
    }

    /*
     * The "main" method of serverThread, this is called when the Thread is
     * started. It first reads the request, constructs the reply. The reply is
     * HTTP-Header and content (constructed by getContent()).
     * 
     * (non-Javadoc)
     * 
     * @see java.lang.Thread#run()
     */
    public void run() {
        try {
            bin = new BufferedReader(new InputStreamReader(sock
                    .getInputStream()));
            bout = new PrintWriter(sock.getOutputStream());
            System.out.println("Receiving");

            String line = " ";
            line = bin.readLine();
            System.out.println(line);
            // Create an URL Object, this way it is easier to access the path
            // and the query.
            URL httpurl = new URL("http", "localhost", "/");
            StringTokenizer theRequest = new StringTokenizer(line);
            if (theRequest.hasMoreTokens()) {
                if (theRequest.nextToken().equals("GET")) {
                    httpurl = new URL("http", "localhost", theRequest
                            .nextToken());
                }
            }

            System.out.println("Stopped Receiving");

            String content = getContent(httpurl);

            // This is the HTTP-Header
            Date now = new Date();
            String response = "HTTP/1.1 200 OK \r\n";
            response += "Server: JBServ/0.0.1 \r\n";
            response += "Date: " + now + "\r\n";
            response += "Content-Length: \r\n";
            response += "Content-Language: de \r\n";
            response += "Content-Type: text/html \r\n";
            response += "Connection: close \r\n\r\n";

            // String response = header + content;
            bout.println(response + content);
            System.out.println("Sending Response\n");
            // Send reply and close socket
            bout.flush();
            sock.close();
        }

        catch (NullPointerException e) {
            if (sock != null) {
                try {
                    sock.close();
                } catch (IOException ex) {
                    ;
                }
            }
        }

        catch (IOException e) {
            System.err.println(e);
        }
    }

}
